<?php
namespace Opencart\Catalog\Controller\Extension\Vtbpay\Payment;

require __DIR__ . '/vendor/autoload.php';

use \Vtbpay\Classes\VtbApi,
    \Opencart\System\Engine\Registry,
    \Opencart\System\Engine\Controller,
    \Opencart\System\Library\Log;

class Vtbpay extends Controller 
{
    /**
     * Constants for log levels
     */
    const LOG_LEVEL_DEBUG = 'DEBUG';
    const LOG_LEVEL_ERROR = 'ERROR';
    
    private $logger;


    /**
     * VtbPay constructor
     *
     * Set up the logging functionality for the VtbPay transactions.
     *
     * @param object $registry The registry of the OpenCart system
     */
    public function __construct(Registry $registry) 
    {
        parent::__construct($registry);
        $this->logger = new Log('vtbpay.log');
        $this->load->language('extension/vtbpay/payment/vtbpay');
    }
    
    
    /**
     * Get VTB API instance
     *
     * Returns an instance of the VTB API using configuration values.
     *
     * @return VtbApi An instance of the VtbApi
     */
    private function getVtbApi(): VtbApi 
    {
        return new VtbApi(
            $this->config->get('payment_vtbpay_client_id'),
            $this->config->get('payment_vtbpay_client_secret'),
            $this->config->get('payment_vtbpay_test_mode'),
            $this->config->get('payment_vtbpay_merchant_authorization')
        );
    }


    /**
     * Index function
     *
     * Prepares data for the checkout view, and returns the rendered HTML.
     *
     * @return string Rendered HTML of the checkout view
     */
    public function index(): string 
    {
        $data['button_confirm'] = $this->language->get('button_confirm');
        $data['action'] = $this->url->link('extension/vtbpay/payment/vtbpay.confirm', '', true);
        $data['logo'] = '/extension/vtbpay/catalog/view/image/payment/vtbpay.png';
        $data['css'] = '/extension/vtbpay/catalog/view/stylesheet/payment/vtbpay.css';
        $data['desc'] = $this->config->get('payment_vtbpay_desc');
        return $this->load->view('extension/vtbpay/payment/vtbpay', $data);
    }


    /**
     * Confirm function
     *
     * Initiates a payment transaction with VTB Pay and redirects the user to the VTB Pay payment page.
     * Logs all VTB Pay transactions.
     *
     * @return void
     */
    public function confirm(): void 
    {
        $order_id = $this->session->data['order_id'] ?? null;
        
        try {
            if (!$order_id) {
                throw new \Exception($this->language->get('error_no_order_id'));
            }

            $this->load->model('checkout/order');

            $order_info = $this->model_checkout_order->getOrder($order_id);
            if (!$order_info) {
                throw new \Exception($this->language->get('error_order_not_found'));
            }
            
            $this->model_checkout_order->addHistory(
                $order_id, 
                1, 
                'orderId: ' . $order_id, 
                true
            );
        
            $return_url = $this->url->link('extension/vtbpay/payment/vtbpay.callback');

            $response = $this->getVtbApi()->getOrderLink(
                $order_info['order_id'], 
                $order_info['email'], 
                time(), 
                $order_info['total'], 
                $return_url
            );
            
            // Logging
            $this->vtbpayLogger(__FUNCTION__ . ' > getOrderLink. ', self::LOG_LEVEL_DEBUG, [
                'arguments' => [
                    'order_id' => $order_info['order_id'], 
                    'email' => $order_info['email'], 
                    'time' => time(), 
                    'total' => $order_info['total'], 
                    'return_url' => $return_url
                ],
                'response' => $response
            ]);

            if (
                !isset($response['object']['payUrl']) || 
                empty($pay_url = $response['object']['payUrl'])
            ) {
                throw new \Exception($this->language->get('error_pay_url_not_available'));
            } 
            
            unset($this->session->data['order_id']);
            
            $this->response->redirect($pay_url);
            
        } catch (\Exception $e) {
            // Handle exception and log error
            $this->vtbpayLogger(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order ID: %s', 
                $e->getMessage(),
                $order_id ?: ''
            ), self::LOG_LEVEL_ERROR);
            $this->session->data['error'] = $e->getMessage();
            $this->response->redirect($this->url->link('checkout/failure', '', true));
        }
    }

    
    /**
     * Callback function
     *
     * Processes the callback from VTB Pay, updates the order status based on the callback data,
     * and prepares and displays a success or failure message to the user.
     *
     * @return void
     */
    public function callback(): void 
    {
        $order_id = $this->request->get['orderId'] ?? null;
        
        try {
            if (!$order_id) {
                throw new \Exception($this->language->get('error_no_order_id'));
            }

            $this->load->model('checkout/order');

            $order_info = $this->model_checkout_order->getOrder($order_id);
            if (!$order_info) {
                throw new \Exception($this->language->get('error_order_not_found'));
            }

            // Check if the order belongs to the current authorized user
            if ($order_info['customer_id'] != $this->customer->getId()) {
                throw new \Exception($this->language->get('error_unauthorized_order_access'));
            }
            
            $response = $this->getVtbApi()->getOrderInfo($order_id);
            
            // Logging
            $this->vtbpayLogger(__FUNCTION__ . ' > getOrderInfo. ', self::LOG_LEVEL_DEBUG, [
                'arguments' => [
                    'order_id' => $order_id
                ],
                'response' => $response
            ]);

            if (
                !isset($response['object']['status']['value']) || 
                $response['object']['status']['value'] !== 'PAID'
            ) {
                throw new \Exception($this->language->get('error_payment_not_paid'));
            }
                
            $this->model_checkout_order->addHistory(
                $order_id, 
                $this->config->get('payment_vtbpay_transaction_end_status')
            );
            $this->cart->clear();
            $this->response->redirect($this->url->link('checkout/success'));
            
        } catch (\Exception $e) {
            // Handle exception and log error
            $this->vtbpayLogger(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order ID: %s', 
                $e->getMessage(),
                $order_id ?: ''
            ), self::LOG_LEVEL_ERROR);
            $this->session->data['error'] = $e->getMessage();
            $this->response->redirect($this->url->link('checkout/failure'));
        }
    }
    
    
    /** 
     * Logs a message with optional context. Log type can be either DEBUG or ERROR.
     * 
     * @param string $message The message to log
     * @param string $type The type of log. Can be either DEBUG or ERROR
     * @param array $context An optional array of context data to log with the message
     * @param bool $print_r If set to true, the context array will be printed using print_r instead of json_encode
     * @return void
     */
    private function vtbpayLogger(
        $message, 
        $type = '',
        $context = [],
        $print_r = false
    ) {
				if (empty($type)) $type = self::LOG_LEVEL_DEBUG;
        
        $logging = $this->config->get('payment_vtbpay_logging');
        
        if (
            empty($message) || 
            $type !== self::LOG_LEVEL_ERROR &&
            (
		            !$logging
            )
        ) return null;
        
        $message = $type . ' ' . $message;
        
        if (!empty($context) && is_array($context)) {
            $message .= ' Context: '. PHP_EOL;
            if ($print_r) {
                $message .= print_r($context, true);
            }
            else {
                $message .= json_encode($context);
            }
        }
        
        $this->logger->write($message);
    }
}
